@@ -113,7 +113,8 @@ class Database:
                     port=self.port,
                     host=self.host,
                     user=self.username,
-                    password=self.password
+                    password=self.password,
+                    consume_results=True
                 )
             elif self.database_engine == self.POSTGRES:
                 self.dba = self.database_module.connect(
@@ -196,17 +197,19 @@ class Database:
         else:
             return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
 
-    def get_database_result(self, cursor):
+    def get_database_result(self, cursor, counter, limit, skip):
         """
         Get database results
         :param cursor:
+        :param counter:
+        :param limit:
+        :param skip:
         :return:
         """
         columns = [column[0].lower() for column in cursor.description]
         records = cursor.fetchall()
         rows = [dict(zip(columns, row)) for row in records]
-        columns = [column for column in cursor.description]
-        return DatabaseResult(rows, columns, None)
+        return DatabaseResult(rows, columns, None, counter, limit, skip)
 
     def is_json(self, myjson):
         """
@@ -229,6 +232,9 @@ class Database:
         :param int skip: Offset of records to skip
         :return: DatabaseResult
         """
+        # make a statement to count the records
+        sql_count = f"select count(*) as \"count_records\" from ({sql}) as t"
+
         # modify the select statement for limit and skip
         if self.database_engine == self.FIREBIRD:
             sql = f"select first {limit} skip {skip} * from ({sql}) as t"
@@ -240,10 +246,26 @@ class Database:
             sql = f"select * from ({sql}) as t limit {skip},{limit}"
 
         cursor = self.dba.cursor()
+        counter_cursor = self.dba.cursor()
         try:
+            if "?" in sql_count:
+                sql_count = self.parse_place_holders(sql_count)
+                counter_cursor.execute(sql_count, params)
+            else:
+                counter_cursor.execute(sql_count)
+            count_records = counter_cursor.fetchall()
+
+            if len(count_records) > 0:
+                count_records = count_records[0][0]
+            else:
+                count_records = 0
+
+            counter_cursor.close()
+
             sql = self.parse_place_holders(sql)
+
             cursor.execute(sql, params)
-            return self.get_database_result(cursor)
+            return self.get_database_result(cursor, count_records, limit, skip)
         except Exception as e:
             Debug("FETCH ERROR:",  sql, str(e), "params", params, "limit", limit, "skip", skip, Constant.TINA4_LOG_DEBUG)
             return DatabaseResult(None, [], str(e))
@@ -257,7 +279,6 @@ class Database:
         :return: dict : A dictionary containing the single record
         """
         # Calling the fetch method with limit as 1 and returning the result
-        sql = self.parse_place_holders(sql)
         record = self.fetch(sql, params=params, limit=1, skip=skip)
         if record.error is None and record.count == 1:
             data = {}
@@ -300,11 +321,11 @@ class Database:
         try:
             cursor.execute(sql, params)
             if "returning" in sql.lower():
-                return self.get_database_result(cursor)
+                return self.get_database_result(cursor, 1, 1, 0)
             else:
                 # see if we are mysql and if we are insert statement to get the last record
                 if "insert" in sql.lower() and self.database_engine == self.MYSQL:
-                    return DatabaseResult([{"id": cursor.lastrowid}], [], None)
+                    return DatabaseResult([{"id": cursor.lastrowid}], [], None, 1, 1, 0)
 
                 # On success return an empty result set with no error
                 return DatabaseResult(None, [], None)
@@ -399,20 +420,13 @@ class Database:
         :param str table_name: Name of table
         :param None data: List or Dictionary containing the data to be inserted
         :param str primary_key: The name of the primary key of the table
-        :return:
         """
         if isinstance(data, dict):
             data = [data]
 
         if isinstance(data, list):
             columns = ", ".join(data[0].keys())
-            # Checking which database engine is used to generate respective syntax for placeholders
-            if self.database_engine in (self.SQLITE, self.FIREBIRD):
-                placeholders = ", ".join(['?'] * len(data[0]))
-            elif self.database_engine in (self.MYSQL, self.POSTGRES):
-                placeholders = ", ".join(['%s'] * len(data[0]))
-            else:
-                placeholders = ", ".join(['?'] * len(data))
+            placeholders = ", ".join(['?'] * len(data[0]))
 
             sql = f"INSERT INTO {table_name} ({columns}) VALUES ({placeholders})"
 
@@ -439,14 +453,8 @@ class Database:
         Delete data based on table name and filter provided - single or multiple filters
         :param str table_name: Name of table
         :param str filter: Expression for deleting records
-        :return: bool
         """
-        if self.database_engine in (self.SQLITE, self.FIREBIRD):
-            placeholder = "?"
-        elif self.database_engine in (self.MYSQL, self.POSTGRES):
-            placeholder = "%s"
-        else:
-            placeholder = "?"
+        placeholder = "?"
 
         if filter is not None:
             # Updating a single record - record passed in is a dictionary
@@ -487,14 +495,8 @@ class Database:
         :param str table_name: Name of table
         :param None data: List or Dictionary containing the data to be inserted
         :param str primary_key: The name of the primary key of the table
-        :return:
         """
-        if self.database_engine in (self.SQLITE, self.FIREBIRD):
-            placeholder = "?"
-        elif self.database_engine in (self.MYSQL, self.POSTGRES):
-            placeholder = "%s"
-        else:
-            placeholder = "?"
+        placeholder = "?"
 
         if data is not None:
             # Updating a single record - record passed in is a dictionary
