@@ -11,7 +11,6 @@ import sys
 import ast
 import json
 import os
-import tina4_python
 from tina4_python.Constant import TINA4_LOG_ERROR, TINA4_LOG_INFO
 from tina4_python.Debug import Debug
 
@@ -21,8 +20,10 @@ def find_all_sub_classes(a_class):
 
 
 def orm(dba):
+    from tina4_python import root_path
     Debug("Initializing ORM")
-    orm_path = tina4_python.root_path + os.sep + "src" + os.sep + "orm"
+    orm_path = root_path + os.sep + "src" + os.sep + "orm"
+    print("Path", orm_path)
 
     if not os.path.exists(orm_path):
         os.makedirs(orm_path)
@@ -51,6 +52,7 @@ class BaseField:
     value = None
     field_size = None
     decimal_places = None
+    protected_field = False
 
     def get_definition(self):
         return self.column_name.lower() + " not defined"
@@ -101,12 +103,13 @@ class BaseField:
         return float(self.value)
 
     def __init__(self, column_name=None, primary_key=False, default_value=None, auto_increment=False, field_size=None,
-                 decimal_places=None):
+                 decimal_places=None, protected_field=False):
         self.primary_key = primary_key
         self.column_type = None
         if default_value is not None:
             self.default_value = default_value
         self.auto_increment = auto_increment
+        self.protected_field = protected_field
         if field_size is not None:
             self.field_size = field_size
         if decimal_places is not None:
@@ -195,11 +198,16 @@ class ForeignKeyField:
     foreign_key = True
     value = None
     default_value = None
+    protected_field = False
 
-    def __init__(self, field_type=BaseField, references_table=None, column_name=None):
+    def __init__(self, field_type=BaseField, references_table=None, column_name=None, default_value = None, protected_field=False):
         self.field_type = field_type
         self.references_table = references_table
         self.references_column = field_type.column_name
+        self.default_value = default_value
+        self.auto_increment = False
+        self.protected_field = protected_field
+
         if column_name is None:
             frame = inspect.stack()[1]
             # Parse python syntax of the assignment line
@@ -233,7 +241,7 @@ class ORM:
     __dba__ = None
     __field_definitions__ = {}
 
-    def __get_table_name__(self, name):
+    def __get_snake_case_name__(self, name):
         """
         Gets the table name
         :param name:
@@ -241,15 +249,15 @@ class ORM:
         """
         if "_" in name:
             return name.lower()
-        table_name = ""
+        snake_case_name = ""
         counter = 0
         for c in name:
             if c.isupper() and counter > 0:
-                table_name = table_name + "_" + c.lower()
+                snake_case_name = snake_case_name + "_" + c.lower()
             else:
-                table_name = table_name + c.lower()
+                snake_case_name = snake_case_name + c.lower()
             counter += 1
-        return table_name
+        return snake_case_name
 
     def __init__(self, init_object=None, table_name=None):
         # save the initial declarations
@@ -257,7 +265,7 @@ class ORM:
         self.__field_definitions__ = {}
         for key in dir(self):
             if not key.startswith('__') and not key.startswith('_') and key not in ['save', 'load', 'delete', 'to_json',
-                                                                                    'to_dict']:
+                                                                                    'to_dict', 'create_table', 'select']:
                 self.__field_definitions__[key] = getattr(self, key)
                 counter += 1
 
@@ -267,13 +275,13 @@ class ORM:
         class_name = self.__class__.__name__
         if self.__table_name__ is None:
             if table_name is None:
-                self.__table_name__ = self.__get_table_name__(class_name)
+                self.__table_name__ = self.__get_snake_case_name__(class_name)
             else:
                 self.__table_name__ = table_name.lower()
         if init_object is not None:
             self.__populate_orm(init_object)
 
-        Debug("Checking for ", self.__table_name__, TINA4_LOG_INFO)
+        Debug("Checking for", self.__table_name__, TINA4_LOG_INFO)
         if self.__dba__:
             self.__table_exists = self.__dba__.table_exists(self.__table_name__)
             if not self.__table_exists:
@@ -283,17 +291,27 @@ class ORM:
                 with open(filename, "w") as f:
                     f.write(sql)
                     f.close()
-            Debug("Table Exists ", self.__table_exists, TINA4_LOG_INFO)
+            Debug("Table Exists", self.__table_exists, TINA4_LOG_INFO)
+        else:
+            self.__table_exists = False
 
     def __populate_orm(self, init_object):
+        """
+        Populates an ORM object from an input object, also transforms camel case objects to snake case ...
+        :param init_object:
+        :return:
+        """
         if isinstance(init_object, str):
             init_object = json.loads(init_object)
 
         for key, value in init_object.items():
-            if key in self.__field_definitions__:
-                field_value = self.__field_definitions__[key]
+            snake_case_name = self.__get_snake_case_name__(key)
+            if  snake_case_name in self.__field_definitions__:
+                field_value = self.__field_definitions__[snake_case_name]
                 field_value.value = value
-                setattr(self, key, field_value)
+                if hasattr(self, snake_case_name):
+                    setattr(self, snake_case_name, field_value)
+
 
     def __get_primary_keys(self):
         primary_keys = []
@@ -311,12 +329,16 @@ class ORM:
         # print(self.__field_definitions__.items(), self.__dict__)
         for key, value in self.__field_definitions__.items():
             current_value = getattr(self, key)
+
             if (isinstance(current_value, ForeignKeyField) or isinstance(current_value, IntegerField) or isinstance(current_value,DateTimeField)
                     or isinstance(current_value, BlobField) or isinstance(current_value, TextField) or isinstance(current_value, StringField)):
+                if current_value.protected_field:
+                    continue
+
                 if current_value.value is not None:
                     data[key] = current_value.value
                 else:
-                    if value.auto_increment:
+                    if not isinstance(current_value, ForeignKeyField) and value.auto_increment:
                         data[key] = self.__dba__.get_next_id(table_name=self.__table_name__, column_name=value.column_name)
                     else:
                         data[key] = current_value.default_value
@@ -328,7 +350,7 @@ class ORM:
         return self.to_json()
 
     def __create_table__(self, table_name, execute=False):
-        sql = "create table " + self.__table_name__ + " ("
+        sql = "create table " + table_name + " ("
         counter = 0
         for field, field_definition in self.__field_definitions__.items():
             if counter > 0:
@@ -347,7 +369,53 @@ class ORM:
         else:
             return sql
 
+    def create_table(self):
+        """
+        Creates the table for the ORM structure
+        :return:
+        """
+        self.__dba__.create_table(self.__table_name__, True)
+
+    def select (self, column_names="*", filter="", params=[], join="", having="", order_by="", limit=10, skip=0):
+        """
+        Selects an array of records based on the ORM object
+        :param filter:
+        :param params:
+        :param order_by:
+        :param limit:
+        :param skip:
+        :return:
+        """
+        if isinstance(column_names, str) and column_names != "*" and column_names != "":
+            column_names = column_names.split(',')
+
+        if column_names == "":
+            column_names = "*"
+
+        if isinstance(order_by, str) and order_by != "":
+            order_by = order_by.split(',')
+
+        sql = "select " + ",\n".join(column_names) + "\nfrom " + self.__table_name__ +" as t "
+        if join != "":
+            sql += "\n"+join
+        if filter != "":
+            sql += "\nwhere " + filter
+        if having != "":
+            sql += having
+
+        if len(order_by) > 0:
+            sql += "\norder by " + ",".join(order_by)
+
+        records = self.__dba__.fetch(sql, params=params, limit=limit, skip=skip)
+        return records
+
     def load(self, query="", params=[]):
+        """
+        Loads a single record into the object based on the primary key or query if query is set
+        :param query:
+        :param params:
+        :return:
+        """
         if not self.__table_exists:
             Debug("ORM: Load Error - Table", self.__table_name__, "does not exist", TINA4_LOG_ERROR)
             return False
@@ -373,7 +441,9 @@ class ORM:
                             field_value = self.__field_definitions__[key]
                             field_value.value = value
                             setattr(self, key, field_value)
-
+                    return True
+                else:
+                    return False
             except Exception as e:
                 Debug("ORM Load Error", str(e), TINA4_LOG_ERROR)
         else:
@@ -401,17 +471,23 @@ class ORM:
             input_params.append(data[key])
             counter += 1
 
-        record = self.__dba__.fetch_one(sql, input_params)
+        try:
 
-        if record["count_records"] == 0:
-            result = self.__dba__.insert(self.__table_name__, data)
-        else:
-            result = self.__dba__.update(self.__table_name__, data)
+            record = self.__dba__.fetch_one(sql, input_params)
 
-        self.__dba__.commit()
+            if record["count_records"] == 0:
+                result = self.__dba__.insert(self.__table_name__, data)
+            else:
+                result = self.__dba__.update(self.__table_name__, data)
 
-        if result:
-            self.load()
+            self.__dba__.commit()
+
+            if result:
+                self.load()
+                return True
+        except Exception as e:
+            Debug.error("Error saving", str(e))
+            return False
 
         return result
 
@@ -435,7 +511,10 @@ class ORM:
         if self.__dba__ is not None:
             result = self.__dba__.delete(sql, params)
 
-        if result.error is not None:
+        if not result:
+            self.__dba__.rollback()
             return False
         else:
+            self.__dba__.commit()
             return True
+        pass
